/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ApiRootLinkRel, ApiRootResource, ApiRootService } from '@alfa-client/api-root-shared';
import { NavigationService } from '@alfa-client/navigation-shared';
import { ResourceRedirectService } from '@alfa-client/resource-redirect-shared';
import {
  ResourceRepository,
  StateResource,
  createEmptyStateResource,
  createStateResource,
  toResourceUri,
} from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { Messages, SnackBarService } from '@alfa-client/ui';
import { Resource, ResourceUri } from '@ngxp/rest';
import { of, throwError } from 'rxjs';
import { createApiRootResource } from '../../../api-root-shared/test/api-root';
import { createDummyResource } from '../../../tech-shared/test/resource';
import { ResourceLinkRel } from './resource-linkrel';

describe('ResourceRedirectService', () => {
  let service: ResourceRedirectService;
  let apiRootService: Mock<ApiRootService>;
  let repository: Mock<ResourceRepository>;
  let snackbarService: Mock<SnackBarService>;
  let navigationService: Mock<NavigationService>;

  const resourceUri: ResourceUri = 'some-uri';

  beforeEach(() => {
    apiRootService = mock(ApiRootService);
    repository = mock(ResourceRepository);
    snackbarService = mock(SnackBarService);
    navigationService = mock(NavigationService);

    service = new ResourceRedirectService(
      useFromMock(apiRootService),
      useFromMock(repository),
      useFromMock(snackbarService),
      useFromMock(navigationService),
    );
  });

  describe('redirectToResource', () => {
    it('should call getResolvedResource', () => {
      const resource: Resource = createDummyResource([ResourceLinkRel.VORGANG]);
      service.getResolvedResource = jest.fn().mockReturnValue(of(resource));

      service.redirectToResource(resourceUri);

      expect(service.getResolvedResource).toHaveBeenCalled();
    });

    it('should call navigateToResource on success', () => {
      const resource: Resource = createDummyResource([ResourceLinkRel.VORGANG]);
      service.getResolvedResource = jest.fn().mockReturnValue(of(resource));
      service.navigateToResource = jest.fn();

      service.redirectToResource(resourceUri);

      expect(service.navigateToResource).toHaveBeenCalled();
    });

    it('should call handleResolveError on error', () => {
      service.getResolvedResource = jest.fn().mockReturnValue(throwError(() => new Error('some error')));
      service.handleResolveError = jest.fn();

      service.redirectToResource(resourceUri);

      expect(service.handleResolveError).toHaveBeenCalled();
    });
  });

  describe('getResolvedResource', () => {
    it('should call apiRootService getApiRoot', () => {
      const apiRoot: StateResource<ApiRootResource> = createStateResource(createApiRootResource());
      apiRootService.getApiRoot.mockReturnValue(of(apiRoot));

      service.getResolvedResource(resourceUri).subscribe(() => {
        expect(apiRootService.getApiRoot).toHaveBeenCalled();
      });
    });

    it('should return null observable on empty api root resource', () => {
      apiRootService.getApiRoot.mockReturnValue(of(createEmptyStateResource()));

      service.getResolvedResource(resourceUri).subscribe((resolvedResource: Resource) => {
        expect(resolvedResource).toBeNull();
      });
    });

    it('should call repository getResource with result from buildResolveUri', () => {
      const apiRoot: StateResource<ApiRootResource> = createStateResource(createApiRootResource([ApiRootLinkRel.RESOURCE]));
      apiRootService.getApiRoot.mockReturnValue(of(apiRoot));
      service.buildResolveUri = jest.fn().mockReturnValue('some-uri');

      service.getResolvedResource(resourceUri).subscribe(() => {
        expect(repository.getResource).toHaveBeenCalledWith('some-uri');
      });
    });

    it('should return null observable on repository error', () => {
      const apiRoot: StateResource<ApiRootResource> = createStateResource(createApiRootResource([ApiRootLinkRel.RESOURCE]));
      apiRootService.getApiRoot.mockReturnValue(of(apiRoot));
      repository.getResource.mockReturnValue(throwError(() => new Error('some error')));

      service.getResolvedResource(resourceUri).subscribe((resolvedResource: Resource) => {
        expect(resolvedResource).toBeNull();
      });
    });

    it('should return resolved resource', () => {
      const resource: Resource = createDummyResource([ResourceLinkRel.VORGANG]);
      const apiRoot: StateResource<ApiRootResource> = createStateResource(createApiRootResource([ApiRootLinkRel.RESOURCE]));
      apiRootService.getApiRoot.mockReturnValue(of(apiRoot));
      repository.getResource.mockReturnValue(of(resource));

      service.getResolvedResource(resourceUri).subscribe((resolvedResource: Resource) => {
        expect(resolvedResource).toEqual(resource);
      });
    });
  });

  describe('buildResolveUri', () => {
    it('should build resolve uri', () => {
      const apiRootResource: ApiRootResource = createApiRootResource([ApiRootLinkRel.RESOURCE]);
      const resolveUri: ResourceUri = service.buildResolveUri(apiRootResource, resourceUri);

      expect(resolveUri).toBeDefined();
    });

    it('should throw error on missing resource uri', () => {
      const apiRootResource: ApiRootResource = createApiRootResource();

      expect(() => service.buildResolveUri(apiRootResource, resourceUri)).toThrow();
    });
  });

  describe('handleResolveError', () => {
    it('should call snackbarService showInfo', () => {
      service.handleResolveError();

      expect(snackbarService.showInfo).toHaveBeenCalledWith(Messages.HTTP_STATUS_RESOURCE_NOT_FOUND);
    });

    it('should call navigationService navigateToVorgangList', () => {
      service.handleResolveError();

      expect(navigationService.navigateToVorgangList).toHaveBeenCalled();
    });
  });

  describe('navigateToResource', () => {
    it('should call navigationService navigateToVorgang', () => {
      const resource: Resource = createDummyResource([ResourceLinkRel.VORGANG]);
      const resourceUri: ResourceUri = toResourceUri(resource, ResourceLinkRel.VORGANG);

      service.navigateToResource(resource);

      expect(navigationService.navigateToVorgang).toHaveBeenCalledWith(resourceUri);
    });

    it('should not call navigationService on missing resource', () => {
      service.navigateToResource(createDummyResource());

      expect(navigationService.navigateToVorgang).not.toHaveBeenCalled();
    });
  });
});
