/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ApiRootLinkRel, ApiRootResource } from '@alfa-client/api-root-shared';
import { HttpErrorHandler, TechSharedModule } from '@alfa-client/tech-shared';
import { mock, mockClass, useFromMock } from '@alfa-client/test-utils';
import {
  VorgangHeaderLinkRel,
  VorgangResource,
  VorgangWithEingangLinkRel,
  VorgangWithEingangResource,
} from '@alfa-client/vorgang-shared';
import { ResourceFactory } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import {
  createVorgangResource,
  createVorgangWithEingangResource,
} from 'libs/vorgang-shared/test/vorgang';
import { createUserProfileListResource, createUserProfileResource } from '../../test/user-profile';
import { UserProfileListResource, UserProfileResource } from './user-profile.model';
import { UserProfileRepository } from './user-profile.repository';

describe('User profile repository', () => {
  let repository: UserProfileRepository;

  const resourceFactory = mock(ResourceFactory);
  const resourceWrapper = { get: jest.fn() };

  const vorgang: VorgangResource = createVorgangResource();
  const userProfile: UserProfileResource = createUserProfileResource();

  beforeEach(() => {
    const classMock = mockClass(TechSharedModule);
    classMock.injector = <any>{ get: () => useFromMock(mock(HttpErrorHandler)) };

    repository = new UserProfileRepository(useFromMock(resourceFactory));

    resourceFactory.from.mockReturnValue(resourceWrapper);
    resourceFactory.fromId.mockReturnValue(resourceWrapper);
    resourceWrapper.get.mockReturnValue(cold('a', { a: userProfile }));
  });

  it('should be created', () => {
    expect(repository).toBeTruthy();
  });

  describe('get user profile', () => {
    const linkRel: string = VorgangHeaderLinkRel.ASSIGNED_TO;

    it('should call resource factory', () => {
      repository.getUserProfile(vorgang, linkRel);

      expect(resourceFactory.from).toHaveBeenCalledWith(vorgang);
    });

    it('should get', () => {
      repository.getUserProfile(vorgang, linkRel);

      expect(resourceWrapper.get).toHaveBeenCalledWith(linkRel);
    });

    it('should return result', () => {
      const result = repository.getUserProfile(vorgang, linkRel);

      expect(result).not.toBeNull();
      expect(result).toBeObservable(hot('a', { a: userProfile }));
    });
  });

  describe('search', () => {
    const searchBy: string = 'SearchMe';
    const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource([
      VorgangWithEingangLinkRel.SEARCH_USER_PROFILES,
    ]);
    const userProfileListResource: UserProfileListResource = createUserProfileListResource();

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(cold('a', { a: userProfileListResource }));
    });

    it('should call resource factory', () => {
      repository.search(vorgangWithEingang, searchBy);

      expect(resourceFactory.fromId).toHaveBeenCalled();
    });

    it('should build search-url', () => {
      repository.buildSearchByUrl = jest.fn();

      repository.search(vorgangWithEingang, searchBy);

      expect(repository.buildSearchByUrl).toHaveBeenCalledWith(vorgangWithEingang, searchBy);
    });

    it('should return result', () => {
      const result = repository.search(vorgangWithEingang, searchBy);

      expect(result).not.toBeNull();
      expect(result).toBeObservable(hot('a', { a: userProfileListResource }));
    });
  });

  describe('getCurrentUser', () => {
    const apiRoot: ApiRootResource = createApiRootResource();

    it('should call resource Factory', () => {
      repository.getCurrentUser(apiRoot);

      expect(resourceFactory.from).toHaveBeenCalledWith(apiRoot);
    });

    it('should call get', () => {
      repository.getCurrentUser(apiRoot);

      expect(resourceWrapper.get).toHaveBeenCalledWith(ApiRootLinkRel.CURRENT_USER);
    });

    it('should return result', () => {
      const result = repository.getCurrentUser(apiRoot);

      expect(result).not.toBeNull();
      expect(result).toBeObservable(hot('a', { a: userProfile }));
    });
  });
});
