/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidResource, BescheidService } from '@alfa-client/bescheid-shared';
import { BinaryFileAttachmentContainerComponent } from '@alfa-client/binary-file';
import { BinaryFileResource } from '@alfa-client/binary-file-shared';
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule, UntypedFormBuilder } from '@angular/forms';
import { FileUploadEditorComponent } from '@ods/component';
import { MockComponent } from 'ng-mocks';
import { EMPTY, Observable, Subscription, of } from 'rxjs';
import { createBescheidStateResource } from '../../../../../../../../../bescheid-shared/src/test/bescheid';
import { createBinaryFileResource } from '../../../../../../../../../binary-file-shared/test/binary-file';
import { createFile } from '../../../../../../../../../tech-shared/test/file';
import { singleColdCompleted } from '../../../../../../../../../tech-shared/test/marbles';
import { BescheidenFormService } from '../../../../bescheiden.formservice';
import { VorgangDetailBescheidenAttachmentHochladenComponent } from './vorgang-detail-bescheiden-attachment-hochladen.component';

describe('VorgangDetailBescheidenDokumentHochladenComponent', () => {
  let component: VorgangDetailBescheidenAttachmentHochladenComponent;
  let fixture: ComponentFixture<VorgangDetailBescheidenAttachmentHochladenComponent>;
  const selfLink: string = 'self';

  let bescheidService: Mock<BescheidService>;
  let formService: BescheidenFormService;

  beforeEach(async () => {
    bescheidService = mock(BescheidService);
    bescheidService.getAttachments.mockReturnValue(EMPTY);

    formService = new BescheidenFormService(new UntypedFormBuilder(), useFromMock(bescheidService));

    await TestBed.configureTestingModule({
      declarations: [
        VorgangDetailBescheidenAttachmentHochladenComponent,
        MockComponent(BinaryFileAttachmentContainerComponent),
        MockComponent(FileUploadEditorComponent),
      ],
      imports: [ReactiveFormsModule],
      providers: [
        {
          provide: BescheidenFormService,
          useValue: formService,
        },
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailBescheidenAttachmentHochladenComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    let loadExistingAttachments: jest.Mock;
    let subscribeToFileDeletion: jest.Mock;

    beforeEach(() => {
      loadExistingAttachments = component.loadExistingAttachments = jest.fn();
      subscribeToFileDeletion = component.subscribeToFileDeletion = jest.fn();
    });

    it('should load existing attachments', () => {
      component.ngOnInit();

      expect(loadExistingAttachments).toHaveBeenCalled();
    });

    it('should subscribe to file deletion', () => {
      component.ngOnInit();

      expect(subscribeToFileDeletion).toHaveBeenCalled();
    });
  });

  describe('ngOnDestroy', () => {
    it('should unsubscribe from file deletion', () => {
      const subscription = mock(Subscription);
      const getDeleteFile = (formService.getFileDelete = jest.fn());
      const observable = mock(Observable);
      getDeleteFile.mockReturnValue(observable);
      observable.subscribe.mockReturnValue(subscription);
      component.ngOnInit();

      component.ngOnDestroy();

      expect(subscription.unsubscribe).toHaveBeenCalled();
    });
  });

  describe('loadExistingAttachments', () => {
    let binaryFileResource: BinaryFileResource;

    beforeEach(() => {
      binaryFileResource = createBinaryFileResource();
      bescheidService.getAttachments.mockReturnValue(of([binaryFileResource]));
    });

    it('should get attachments', () => {
      component.loadExistingAttachments();

      expect(bescheidService.getAttachments).toHaveBeenCalled();
    });

    it('should add attachments to fileList', () => {
      component.loadExistingAttachments();

      expect(component.fileList).toEqual([binaryFileResource]);
    });
  });

  describe('subscribeToFileDeletion', () => {
    let getFileDelete: jest.Mock;
    let getFileDeleteObservable: Mock<Observable<BinaryFileResource>>;

    beforeEach(() => {
      getFileDeleteObservable = mock(Observable);
      getFileDelete = formService.getFileDelete = jest
        .fn()
        .mockReturnValue(getFileDeleteObservable);
    });

    it('should subscribe to file deletion', () => {
      component.subscribeToFileDeletion();

      expect(getFileDelete).toHaveBeenCalled();
    });
  });

  describe('deleteFile', () => {
    let binaryFileResource: BinaryFileResource;

    beforeEach(() => {
      binaryFileResource = createBinaryFileResource();
    });

    it('should remove file from list', () => {
      component.fileList = [binaryFileResource];

      component.deleteFile(binaryFileResource);

      expect(component.fileList).toEqual([]);
    });
  });

  describe('getFileLinkList', () => {
    it('should return url list', () => {
      const binaryFileResource = createBinaryFileResource([selfLink]);
      component.fileList = [binaryFileResource];

      const uriList = component.getFileLinkList();

      expect(uriList[0]).toEqual(binaryFileResource._links.self.href);
    });
  });

  describe('uploadFile', () => {
    const bescheidDraftStateResource: StateResource<BescheidResource> =
      createBescheidStateResource();
    const file: File = createFile();
    const binaryFileStateResource: StateResource<BinaryFileResource> = createStateResource(
      createBinaryFileResource(),
    );

    beforeEach(() => {
      bescheidService.uploadAttachment.mockReturnValue(of(binaryFileStateResource));
      component.bescheidDraftStateResource = bescheidDraftStateResource;
    });

    it('should upload attachment', () => {
      component.uploadFile(file);

      component.uploadInProgress$.subscribe();

      expect(bescheidService.uploadAttachment).toHaveBeenCalledWith(
        bescheidDraftStateResource.resource,
        file,
      );
    });

    it('should add to file list', () => {
      component.uploadFile(file);

      component.uploadInProgress$.subscribe();

      expect(component.fileList).toEqual([binaryFileStateResource.resource]);
    });

    it('should emit', () => {
      component.uploadFile(file);

      expect(component.uploadInProgress$).toBeObservable(
        singleColdCompleted(binaryFileStateResource),
      );
    });
  });
});
