/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidResource, BescheidService } from '@alfa-client/bescheid-shared';
import { BinaryFileResource } from '@alfa-client/binary-file-shared';
import {
  StateResource,
  createEmptyStateResource,
  doOnValidStateResource,
  isNotNil,
} from '@alfa-client/tech-shared';
import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { getUrl } from '@ngxp/rest';
import { Observable, Subscription, first, of, tap } from 'rxjs';
import { BescheidenFormService } from '../../../../bescheiden.formservice';

@Component({
  selector: 'alfa-vorgang-detail-bescheiden-attachment-hochladen',
  templateUrl: './vorgang-detail-bescheiden-attachment-hochladen.component.html',
  styles: [],
})
export class VorgangDetailBescheidenAttachmentHochladenComponent implements OnInit, OnDestroy {
  @Input() bescheidDraftStateResource: StateResource<BescheidResource>;

  uploadInProgress$: Observable<StateResource<BinaryFileResource>>;
  fileList: BinaryFileResource[] = [];
  private deleteFileSubscription: Subscription;

  readonly formServiceClass = BescheidenFormService;

  constructor(
    public readonly formService: BescheidenFormService,
    private readonly bescheidService: BescheidService,
  ) {
    this.uploadInProgress$ = of(createEmptyStateResource<BinaryFileResource>());
  }

  ngOnInit(): void {
    this.loadExistingAttachments();
    this.subscribeToFileDeletion();
  }

  ngOnDestroy(): void {
    if (isNotNil(this.deleteFileSubscription)) this.deleteFileSubscription.unsubscribe();
  }

  loadExistingAttachments(): void {
    this.bescheidService
      .getAttachments()
      .pipe(first())
      .subscribe((attachments) => (this.fileList = attachments));
  }

  subscribeToFileDeletion(): void {
    this.deleteFileSubscription = this.formService
      .getFileDelete()
      .subscribe((binaryFileResource) => this.deleteFile(binaryFileResource));
  }

  deleteFile(binaryFileResource: BinaryFileResource) {
    this.fileList = this.fileList.filter((file) => getUrl(file) !== getUrl(binaryFileResource));
  }

  getFileLinkList(): string[] {
    return this.fileList.map((fileResource: BinaryFileResource) => getUrl(fileResource));
  }

  public uploadFile(file: File): void {
    this.uploadInProgress$ = this.bescheidService
      .uploadAttachment(this.bescheidDraftStateResource.resource, file)
      .pipe(
        tap((stateResource: StateResource<BinaryFileResource>) =>
          doOnValidStateResource(
            stateResource,
            () => (this.fileList = [...this.fileList, stateResource.resource]),
          ),
        ),
      );
  }
}
