/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.callcontext;

import static de.ozgcloud.alfa.common.GrpcUtil.*;

import java.util.Objects;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.RequestAttributes;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.vorgang.grpc.command.GrpcCallContext;
import de.ozgcloud.vorgang.grpc.command.GrpcUser;
import io.grpc.Metadata;
import lombok.extern.log4j.Log4j2;

@Service
@Log4j2
public class ContextService {

	static final String KEY_USER_ID = "USER_ID-bin";
	static final String KEY_USER_NAME = "USER_NAME-bin";
	public static final String KEY_CLIENT_NAME = "CLIENT_NAME-bin";
	public static final String KEY_REQUEST_ID = "REQUEST_ID-bin";
	static final String KEY_ACCESS_LIMITED_ORGAID = "ACCESS_LIMITED_TO_ORGANISATORISCHEEINHEITENID-bin";
	static final String KEY_ACCESS_LIMITED = "ACCESS_LIMITED-bin";

	@Autowired
	private ApplicationContext context;

	@Autowired
	private CurrentUserService userService;
	@Autowired(required = false)
	private RequestAttributes reqAttributes;

	public GrpcCallContext createCallContext() {
		return GrpcCallContext.newBuilder()
				.setClient(context.getId())
				.setUser(createUser())
				.setRequestId(getRequestId())
				.build();
	}

	public String getClientName() {
		return context.getId();
	}

	private GrpcUser createUser() {
		var user = userService.getUser();

		return GrpcUser.newBuilder()
				.setId(user.getId().toString())
				.setName(user.getFullName())
				.build();
	}

	public Metadata buildCallContextMetadata() {
		var user = userService.getUser();
		var metadata = buildCallContextWithoutUser();

		Optional.ofNullable(user.getId()).map(id -> id.toString().getBytes())
				.ifPresentOrElse(bytes -> metadata.put(createKeyOf(KEY_USER_ID), bytes), () -> LOG.warn("Missing value 'userid'"));
		Optional.ofNullable(user.getFullName()).map(String::getBytes)
				.ifPresentOrElse(bytes -> metadata.put(createKeyOf(KEY_USER_NAME), bytes), () -> LOG.warn("Missing value 'fullname'"));

		user.getOrganisationseinheitIds().stream().map(String::getBytes)
				.forEach(bytes -> metadata.put(createKeyOf(KEY_ACCESS_LIMITED_ORGAID), bytes));

		return metadata;
	}

	public Metadata buildCallContextWithoutUser() {
		var metadata = new Metadata();

		Optional.ofNullable(context.getId()).map(String::getBytes)
				.ifPresentOrElse(bytes -> metadata.put(createKeyOf(KEY_CLIENT_NAME), bytes), () -> LOG.warn("Missing value 'client name'"));

		metadata.put(createKeyOf(KEY_REQUEST_ID), getRequestId().getBytes());
		metadata.put(createKeyOf(KEY_ACCESS_LIMITED), isOrganisationEinheitenIdCheckNecessary());

		return metadata;
	}

	private byte[] isOrganisationEinheitenIdCheckNecessary() {
		var hasRoleWithNoCheckNecessary = userService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)
				|| userService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER);

		return Boolean.toString(!hasRoleWithNoCheckNecessary).getBytes();
	}

	String getRequestId() {
		return Objects.isNull(reqAttributes) ? "- no request scope -" : reqAttributes.getRequestId();
	}
}