/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.apache.commons.lang3.ArrayUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.callcontext.ContextService;
import de.ozgcloud.alfa.vorgang.RedirectRequestTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.wiedervorlage.WiedervorlageTestFactory;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectTestFactory;
import de.ozgcloud.vorgang.grpc.command.CommandServiceGrpc.CommandServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.command.GrpcCallContext;
import de.ozgcloud.vorgang.grpc.command.GrpcCommand;
import de.ozgcloud.vorgang.grpc.command.GrpcCommandResponse;
import de.ozgcloud.vorgang.grpc.command.GrpcCommandsResponse;
import de.ozgcloud.vorgang.grpc.command.GrpcCreateCommandRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcExistsPendingCommandsRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcExistsPendingCommandsResponse;
import de.ozgcloud.vorgang.grpc.command.GrpcFindCommandsRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcGetCommandRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcGetPendingCommandsRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcGetPendingCommandsResponse;
import de.ozgcloud.vorgang.grpc.command.GrpcRevokeCommandRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcSetCommandExecutedRequest;

class CommandRemoteServiceTest {

	@Spy
	@InjectMocks
	private CommandRemoteService service;
	@Mock
	private CommandServiceBlockingStub commandServiceStub;
	@Mock
	private CommandMapper mapper;
	@Mock
	private CommandBodyMapper bodyMapper;
	@Mock
	private GrpcObjectMapper grpcObjectMapper;
	@Mock
	private ContextService contextService;

	private GrpcCallContext grpcCallContext = GrpcCallContext.newBuilder().build();

	@Nested
	@DisplayName("Create CreateCommand")
	class TestCreateCommand {

		private GrpcCreateCommandRequest request = GrpcCreateCommandRequest.newBuilder().build();
		private CreateCommand createCommand = CommandTestFactory.createCreateCommand();

		@Nested
		class CallGrpcEndpoint {

			private Command resultCommand = CommandTestFactory.create();

			@BeforeEach
			void initTest() {
				doReturn(request).when(service).buildCreateCommandRequest(any());
				when(commandServiceStub.createCommand(any())).thenReturn(GrpcCommandResponse.newBuilder().build());
				when(mapper.toCommand(any())).thenReturn(resultCommand);
			}

			@Test
			void shouldCreateRequest() {
				service.createCommand(createCommand);

				verify(service).buildCreateCommandRequest(createCommand);
			}

			@Test
			void shouldCallServiceStub() {
				service.createCommand(createCommand);

				verify(commandServiceStub).createCommand(request);
			}

			@Test
			void shouldReturnResultCommand() {
				var result = service.createCommand(createCommand);

				assertThat(result).isSameAs(resultCommand);
			}
		}

		@DisplayName("build request")
		@Nested
		class TestBuildRequest {

			@BeforeEach
			void initTest() {
				when(contextService.createCallContext()).thenReturn(grpcCallContext);
				when(grpcObjectMapper.fromMap(any())).thenReturn(GrpcObjectTestFactory.create());
			}

			@Test
			void shouldCreateRequest() {
				var request = buildRequest();

				assertThat(request).isNotNull();
			}

			@Test
			void shouldHaveOrderString() {
				var request = buildRequest();

				assertThat(request.getOrderString()).isEqualTo(CommandTestFactory.ORDER);
			}

			@Test
			void shouldHaveCallContext() {
				var request = buildRequest();

				assertThat(request.getCallContext()).isSameAs(grpcCallContext);
			}

			@Test
			void shouldHaveRelationId() {
				var request = buildRequest();

				assertThat(request.getRelationId()).isEqualTo(CommandTestFactory.RELATION_ID);
			}

			@Test
			void shouldHaveVorgangId() {
				var request = buildRequest();

				assertThat(request.getVorgangId()).isEqualTo(CommandTestFactory.VORGANG_ID);
			}

			@Test
			void shouldCallBodyMapper() {
				buildRequest();

				verify(bodyMapper).fromObjectToMap(any(CommandBody.class));
			}

			@Test
			void shouldCallObjectMapper() {
				buildRequest();

				verify(grpcObjectMapper).fromMap(any());
			}

			@Test
			void shouldHaveVersion() {
				var request = buildRequest();

				assertThat(request.getRelationVersion()).isEqualTo(CommandTestFactory.RELATION_VERSION);
			}

			@Test
			void shouldHaveRedirectRequest() {
				var command = CommandTestFactory.createCreateCommandBuilder().body(null).redirectRequest(RedirectRequestTestFactory.create()).build();

				var request = service.buildCreateCommandRequest(command);

				assertThat(request.getRedirectRequest().getPassword()).isEqualTo(RedirectRequestTestFactory.PASSWORD_AS_STR);
				assertThat(request.getRedirectRequest().getEmail()).isEqualTo(RedirectRequestTestFactory.EMAIL);
			}

			@ParameterizedTest
			@EnumSource
			void shouldHaveOrder(CommandOrder order) {
				var request = service.buildCreateCommandRequest(CommandTestFactory.createCreateCommandBuilder().order(order.name()).build());

				assertThat(request.getOrderString()).isEqualTo(order.name());
			}

			private GrpcCreateCommandRequest buildRequest() {
				return service.buildCreateCommandRequest(
						CommandTestFactory.createCreateCommandBuilder().body(WiedervorlageTestFactory.create()).build());
			}
		}

		@DisplayName("build forward request")
		@Nested
		class TestBuildForwardRequest {

			@Test
			void shouldNotSetPassword() {
				var request = RedirectRequestTestFactory.createBuilder().password(ArrayUtils.EMPTY_CHAR_ARRAY).build();

				var result = service.buildForwardRequest(request);

				assertThat(result.getPassword()).isBlank();
			}

			@Test
			void shouldSetPassword() {
				var request = RedirectRequestTestFactory.createBuilder().password(RedirectRequestTestFactory.PASSWORD).build();

				var result = service.buildForwardRequest(request);

				assertThat(result.getPassword()).isEqualTo(RedirectRequestTestFactory.PASSWORD_AS_STR);
			}

			@Test
			void shouldSetEmail() {
				var request = RedirectRequestTestFactory.create();

				var result = service.buildForwardRequest(request);

				assertThat(result.getEmail()).isEqualTo(RedirectRequestTestFactory.EMAIL);
			}
		}
	}

	@Nested
	@DisplayName("Revoke Command")
	class TestRevokeCommand {

		@Nested
		class CreateRequest {

			@BeforeEach
			void initTest() {
				when(contextService.createCallContext()).thenReturn(grpcCallContext);
			}

			@Test
			void shouldReturnRequest() {
				var request = service.createRevokeRequest(CommandTestFactory.ID);

				assertThat(request).isNotNull().isInstanceOf(GrpcRevokeCommandRequest.class);
			}

			@Test
			void shouldHaveCallContext() {
				var request = service.createRevokeRequest(CommandTestFactory.ID);

				assertThat(request.getContext()).isNotNull().isSameAs(grpcCallContext);
				verify(contextService).createCallContext();
			}

			@Test
			void shouldHaveCommandId() {
				var request = service.createRevokeRequest(CommandTestFactory.ID);

				assertThat(request.getId()).isEqualTo(CommandTestFactory.ID);
			}
		}

		@Nested
		class SendRequest {

			private GrpcRevokeCommandRequest request = GrpcRevokeCommandRequest.newBuilder().build();

			@BeforeEach
			void initTest() {
				doReturn(request).when(service).createRevokeRequest(anyString());
				when(commandServiceStub.revokeCommand(any())).thenReturn(GrpcCommandResponse.newBuilder().build());
			}

			@Test
			void shouldCreateRequest() {
				service.revokeCommand(CommandTestFactory.ID);

				verify(service).createRevokeRequest(CommandTestFactory.ID);
			}

			@Test
			void shouldCallGrpc() {
				service.revokeCommand(CommandTestFactory.ID);

				verify(commandServiceStub).revokeCommand(request);
			}

			@Test
			void shouldMapCommand() {
				service.revokeCommand(CommandTestFactory.ID);

				verify(mapper).toCommand(any());
			}
		}
	}

	@Nested
	@DisplayName("Get command")
	class TestGetCommand {

		@Nested
		class CreateRequest {

			@BeforeEach
			void initTest() {
				when(contextService.createCallContext()).thenReturn(grpcCallContext);
			}

			@Test
			void shouldReturnRequest() {
				var request = service.createGetCommandRequest(CommandTestFactory.ID);

				assertThat(request).isNotNull().isInstanceOf(GrpcGetCommandRequest.class);
			}

			@Test
			void shouldHaveCallContext() {
				var request = service.createGetCommandRequest(CommandTestFactory.ID);

				assertThat(request.getContext()).isNotNull().isSameAs(grpcCallContext);
				verify(contextService).createCallContext();
			}

			@Test
			void shouldHaveCommandId() {
				var request = service.createGetCommandRequest(CommandTestFactory.ID);

				assertThat(request.getId()).isEqualTo(CommandTestFactory.ID);
			}
		}

		@DisplayName("send request")
		@Nested
		class SendRequest {

			private GrpcGetCommandRequest request = GrpcGetCommandRequest.newBuilder().build();

			@BeforeEach
			void initTest() {
				doReturn(request).when(service).createGetCommandRequest(anyString());
				when(commandServiceStub.getCommand(any())).thenReturn(GrpcCommand.newBuilder().build());
			}

			@Test
			void shouldCreateRequest() {
				service.getCommand(CommandTestFactory.ID);

				verify(service).createGetCommandRequest(CommandTestFactory.ID);
			}

			@Test
			void shouldCallGrpc() {
				service.getCommand(CommandTestFactory.ID);

				verify(commandServiceStub).getCommand(request);
			}

			@Test
			void shouldMapCommand() {
				service.getCommand(CommandTestFactory.ID);

				verify(mapper).toCommand(any());
			}
		}
	}

	@Nested
	class TestExistsPendingCommands {

		private final GrpcExistsPendingCommandsRequest request = GrpcExistsPendingCommandsRequest.newBuilder()
				.setContext(grpcCallContext)
				.setVorgangId(VorgangHeaderTestFactory.ID).build();

		@BeforeEach
		void mockServiceStub() {
			when(commandServiceStub.existsPendingCommands(any()))
					.thenReturn(GrpcExistsPendingCommandsResponse.newBuilder().setExistsPendingCommands(true).build());

			when(contextService.createCallContext()).thenReturn(grpcCallContext);
		}

		@Test
		void shouldCallServiceStub() {
			service.existsPendingCommands(VorgangHeaderTestFactory.ID);

			verify(commandServiceStub).existsPendingCommands(request);
		}

		@Test
		void shouldCallContextService() {
			service.existsPendingCommands(VorgangHeaderTestFactory.ID);

			verify(contextService).createCallContext();
		}

		@Test
		void shouldReturnValue() {
			var result = service.existsPendingCommands(VorgangHeaderTestFactory.ID);

			assertThat(result).isTrue();
		}
	}

	@Nested
	class TestFindCommands {

		private final GrpcCommand grpcCommand = GrpcCommand.newBuilder().build();
		private final GrpcCommandsResponse response = GrpcCommandsResponse.newBuilder().addAllCommand(Collections.singleton(grpcCommand)).build();

		@Captor // NOSONAR
		private ArgumentCaptor<GrpcFindCommandsRequest> requestCaptor;

		@BeforeEach
		void init() {
			when(commandServiceStub.findCommands(any())).thenReturn(response);
			when(contextService.createCallContext()).thenReturn(grpcCallContext);
		}

		@Test
		void shouldCallServiceStub() {
			service.findCommands(VorgangHeaderTestFactory.ID, Optional.empty(), Optional.empty());

			verify(commandServiceStub).findCommands(notNull());
		}

		@Test
		void shouldRequestWithVorgangId() {
			service.findCommands(VorgangHeaderTestFactory.ID, Optional.empty(), Optional.empty());

			verify(commandServiceStub).findCommands(requestCaptor.capture());

			assertThat(requestCaptor.getValue().getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldRequestWithStatus() {
			service.findCommands(VorgangHeaderTestFactory.ID, Optional.of(CommandStatus.FINISHED), Optional.empty());

			verify(commandServiceStub).findCommands(requestCaptor.capture());

			assertThat(requestCaptor.getValue().getStatusList()).contains(CommandStatus.FINISHED.name());
		}

		@Test
		void shouldRequestWithOrder() {
			service.findCommands(VorgangHeaderTestFactory.ID, Optional.empty(), Optional.of(CommandOrder.REDIRECT_VORGANG.name()));

			verify(commandServiceStub).findCommands(requestCaptor.capture());

			assertThat(requestCaptor.getValue().getOrderString()).isEqualTo(CommandOrder.REDIRECT_VORGANG.name());
		}

		@Test
		void shouldCallMapper() {
			service.findCommands(VorgangHeaderTestFactory.ID, Optional.empty(), Optional.empty()).toList();

			verify(mapper).toCommand(grpcCommand);
		}
	}

	@Nested
	class TestGetPendingCommands {

		private final GrpcGetPendingCommandsRequest request = GrpcGetPendingCommandsRequest.newBuilder()
				.setContext(grpcCallContext)
				.setVorgangId(VorgangHeaderTestFactory.ID).build();
		private final GrpcCommand grpcCommand = GrpcCommand.newBuilder().build();
		private final Set<GrpcCommand> commandList = Collections.singleton(grpcCommand);
		private final GrpcGetPendingCommandsResponse response = GrpcGetPendingCommandsResponse.newBuilder().addAllCommand(commandList).build();
		private final Command command = CommandTestFactory.create();

		@BeforeEach
		void mockServiceStub() {
			when(commandServiceStub.getPendingCommands(any())).thenReturn(response);
			when(contextService.createCallContext()).thenReturn(grpcCallContext);
		}

		@Test
		void shouldCallServiceStub() {
			service.getPendingCommands(VorgangHeaderTestFactory.ID);

			verify(commandServiceStub).getPendingCommands(request);
		}

		@Test
		void shouldCallContextService() {
			service.getPendingCommands(VorgangHeaderTestFactory.ID);

			verify(contextService).createCallContext();
		}

		@Test
		void shouldCallMapper() {
			var result = service.getPendingCommands(VorgangHeaderTestFactory.ID);
			forceStreamOperationsDone(result);

			verify(mapper).toCommand(grpcCommand);
		}

		@Test
		void shouldReturnValue() {
			when(mapper.toCommand(any())).thenReturn(command);

			var result = service.getPendingCommands(VorgangHeaderTestFactory.ID);

			assertThat(result.toList().get(0)).isEqualTo(command);
		}

		private void forceStreamOperationsDone(Stream<Command> stream) {
			stream.collect(Collectors.toList()).stream();
		}
	}

	@DisplayName("Finish command")
	@Nested
	class TestFinishCommand {

		@Test
		void shouldCallServiceStub() {
			var request = GrpcSetCommandExecutedRequest.newBuilder().setCommandId(CommandTestFactory.ID).build();

			service.finishCommand(CommandTestFactory.ID);

			verify(commandServiceStub).setCommandExecuted(request);
		}
	}
}