/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;

class VorgangServiceTest {

	@Spy
	@InjectMocks // NOSONAR
	private VorgangService service;

	@Mock
	private VorgangRemoteService remoteService;

	@Mock
	private CurrentUserService userService;

	@DisplayName("Find vorgaenge header")
	@Nested
	class TestFindVorgaengeHeader {

		private final FindVorgaengeHeaderRequestCriteria requestCriteria = FindVorgaengeRequestCriteriaTestFactory.create();
		private final FilterCriteria filterCriteria = FilterCriteria.builder().build();

		@Test
		void shouldCallRemoteService() {
			doReturn(requestCriteria).when(service).setOrderBy(any());
			doReturn(filterCriteria).when(service).buildFilterCriteria(any());

			service.findVorgaengeHeader(requestCriteria);

			verify(remoteService).findVorgaengeHeader(requestCriteria, filterCriteria);
		}

		@Test
		void shouldBuildFilterCriteria() {
			when(userService.getUser()).thenReturn(UserProfileTestFactory.create());

			service.findVorgaengeHeader(requestCriteria);

			verify(service).buildFilterCriteria(requestCriteria.getFilterCriteria());
		}

		@DisplayName("order by")
		@Nested
		class TestOrderBy {

			@DisplayName("for " + UserRole.EINHEITLICHER_ANSPRECHPARTNER)
			@Nested
			class TestForEinheitlicherAnsprechpartner {

				@BeforeEach
				void mockUserService() {
					when(userService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)).thenReturn(true);
				}

				@Test
				void shouldSetEaPriority() {
					var request = service.setOrderBy(FindVorgaengeRequestCriteriaTestFactory.create());

					assertThat(request.getOrderBy()).isEqualTo(OrderBy.EA_PRIORITY);
				}
			}

			@DisplayName("for " + UserRole.VERWALTUNG_POSTSTELLE)
			@Nested
			class TestForVerwaltungPoststelle {

				@BeforeEach
				void mockUserService() {
					when(userService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)).thenReturn(false);
					when(userService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)).thenReturn(true);
				}

				@Test
				void shouldSetPriority() {

					var request = service.setOrderBy(FindVorgaengeRequestCriteriaTestFactory.create());

					assertThat(request.getOrderBy()).isEqualTo(OrderBy.PRIORITY);
				}
			}

			@DisplayName("for " + UserRole.VERWALTUNG_USER)
			@Nested
			class TestForVerwaltungUser {

				@BeforeEach
				void mockUserService() {
					when(userService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)).thenReturn(false);
					when(userService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)).thenReturn(false);
				}

				@Test
				void shouldSetToPriorityAsDefault() {
					var request = service.setOrderBy(FindVorgaengeRequestCriteriaTestFactory.createBuilder()
							.filterCriteria(FilterCriteriaTestFactory.createBuilder()
									.hasNextWiedervorlageFrist(false).status(Optional.empty()).build())
							.build());

					assertThat(request.getOrderBy()).isEqualTo(OrderBy.PRIORITY);
				}

				@Test
				void shouldSetToCreatedAtDescOnExistingStatus() {
					var request = service.setOrderBy(FindVorgaengeRequestCriteriaTestFactory.createBuilder()
							.filterCriteria(FilterCriteriaTestFactory.createBuilder()
									.hasNextWiedervorlageFrist(false).build())
							.build());

					assertThat(request.getOrderBy()).isEqualTo(OrderBy.CREATED_AT_DESC);
				}

				@Test
				void shouldSetToNextWiedervorlageFristIfItsTrue() {
					var request = service.setOrderBy(FindVorgaengeRequestCriteriaTestFactory.createBuilder()
							.filterCriteria(FilterCriteriaTestFactory.createBuilder()
									.hasNextWiedervorlageFrist(true)
									.status(Optional.empty()).build())
							.build());

					assertThat(request.getOrderBy()).isEqualTo(OrderBy.NEXT_WIEDERVORLAGE_FRIST);
				}
			}
		}
	}

	@DisplayName("Build filter criteria")
	@Nested
	class TestBuildFilterCriteria {

		@DisplayName("assignedTo")
		@Nested
		class TestAssignedTo {

			@BeforeEach
			void mockUserService() {
				when(userService.hasRole(any())).thenReturn(false);
				when(userService.getUser()).thenReturn(UserProfileTestFactory.create());
			}

			@Test
			void shouldBeSetIfExists() {
				var filterCriteria = buildFilterCriteria(FilterCriteriaTestFactory.create());

				assertThat(filterCriteria.getAssignedTo()).hasValue(UserProfileTestFactory.ID);
			}

			@Test
			void shouldNotBeSetIfNotExist() {
				var filterCriteria = buildFilterCriteria(FilterCriteriaTestFactory.createBuilder().assignedTo(Optional.empty()).build());

				assertThat(filterCriteria.getAssignedTo()).isNotPresent();

			}
		}

		@DisplayName("status")
		@Nested
		class TestStatus {

			@BeforeEach
			void mockUserService() {
				when(userService.hasRole(any())).thenReturn(false);
				when(userService.getUser()).thenReturn(UserProfileTestFactory.create());
			}

			@Test
			void shouldBeSetIfExists() {
				var filterCriteria = buildFilterCriteria(FilterCriteriaTestFactory.create());

				assertThat(filterCriteria.getStatus()).hasValue(VorgangStatus.NEU);
			}

			@Test
			void shouldNotBeSetIfNotExist() {
				var filterCriteria = buildFilterCriteria(FilterCriteriaTestFactory.createBuilder().status(Optional.empty()).build());

				assertThat(filterCriteria.getStatus()).isNotPresent();

			}
		}

		@DisplayName("for " + UserRole.VERWALTUNG_POSTSTELLE)
		@Nested
		class TestForVerwaltungPoststelle {

			@BeforeEach
			void mockUser() {
				when(userService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)).thenReturn(true);
			}

			@Test
			void shouldFillFilterByOrganisationseinheiten() {
				var filterBy = buildFilterCriteria();

				assertThat(filterBy.isFilterByOrganisationseinheitenId()).isFalse();
			}

			@Test
			void shouldFillStatus() {
				var filterBy = buildFilterCriteria();

				assertThat(filterBy.getStatus()).hasValue(VorgangStatus.NEU);
			}
		}

		@DisplayName("for " + UserRole.EINHEITLICHER_ANSPRECHPARTNER)
		@Nested
		class TestForEa {

			@BeforeEach
			void mockUser() {
				when(userService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)).thenReturn(false);
				when(userService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)).thenReturn(true);
			}

			@Test
			void shouldFillFilterByOrganisationseinheiten() {
				var filterBy = buildFilterCriteria();

				assertThat(filterBy.isFilterByOrganisationseinheitenId()).isFalse();
			}
		}

		@DisplayName("for " + UserRole.VERWALTUNG_USER)
		@Nested
		class TestForVerwaltungUser {

			@BeforeEach
			void mockUser() {
				when(userService.hasRole(anyString())).thenReturn(false);
			}

			@Test
			void shouldCallUserService() {
				when(userService.getUser()).thenReturn(UserProfileTestFactory.create());

				buildFilterCriteria();

				verify(userService).getUser();
			}

			@Test
			void shouldFillFilterByOrganisationseinheitenId() {
				when(userService.getUser()).thenReturn(UserProfileTestFactory.create());

				var filterBy = buildFilterCriteria();

				assertThat(filterBy.isFilterByOrganisationseinheitenId()).isTrue();
			}

			@Test
			void shouldFillOrganisationseinheitenId() {
				when(userService.getUser()).thenReturn(UserProfileTestFactory.create());

				var filterBy = buildFilterCriteria();

				assertThat(filterBy.getOrganisationsEinheitenId()).containsExactly(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);
			}
		}

		private FilterCriteria buildFilterCriteria() {
			return buildFilterCriteria(FilterCriteria.builder().build());
		}

		private FilterCriteria buildFilterCriteria(FilterCriteria filterCriteria) {
			return service.buildFilterCriteria(filterCriteria);
		}
	}

	@DisplayName("Find vorgangWithEingang")
	@Nested
	class TestFindVorgangWithEingang {

		private final FilterCriteria filterCriteria = FilterCriteria.builder().build();

		@Test
		void shouldCallRemoteService() {
			doReturn(filterCriteria).when(service).buildFilterCriteria(any());

			service.findVorgangWithEingang(VorgangHeaderTestFactory.ID);

			verify(remoteService).findVorgangWithEingang(VorgangHeaderTestFactory.ID, filterCriteria);
		}

		@Test
		void shouldBuildFilterCriteria() {
			when(userService.getUser()).thenReturn(UserProfileTestFactory.create());

			service.findVorgangWithEingang(VorgangHeaderTestFactory.ID);

			verify(service).buildFilterCriteria(any(FilterCriteria.class));
		}
	}
}